/* CanonUtilFltkWait.cpp
 * Build the wait box and call getPrinterStatus();
 * 
 */
 
/* for OS functions */
#include <FL/x.H>
#include <string.h>

/* needed includes for c source files */
#include "gettext.h"
#include "CanonUtilGlobal.h"
#include "CanonUtilStatus.h"

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include "CanonUtilFltkUtil.h"
#include "CanonUtilFltkMsg.h"
#include "fltkGl.h"

static char *mainTitleLB = { "CanonBjCUtility" };


static Fl_Window *waitWin;
static Fl_Box    *waitTitle;
static Fl_Box    *waitMessage;

static char message[4096];

static void buildMessage()
{
   int i;
   message[0] = '\0';
   strcat(message,getText("status"));
   strcat(message," ");
   switch ( printerState.printerError )
   {
      case PR_ERROR:
         strcat(message,getText("printerError"));
      break;
      case PR_HEAD:
         strcat(message,getText("headError"));
      break;
      case PR_WASTE:
         strcat(message,getText("wasteError"));
      break;
      default:
         if ( printerState.printerError )
            strcat(message,getText("printerBusy"));
         else
            strcat(message,getText("printerOK"));
   }
   strcat(message,"\n");
   
   strcat(message,getText("installedCartridge"));
   strcat(message," ");
   strcat(message,getText(printerState.installedCartridge));
   strcat(message,"\n");

   for ( i = 0; i < 7 ; i++ )
   {
      if ( printerState.colorLevel[i] )
      {
         strcat(message,"\n");
         break;
      }
   }
   if ( i < 8 )
   {
      strcat(message,getText("inkLevel"));
      strcat(message,"\n");
      if ( printerState.colorLevel[BLACK] )
      {
         strcat(message,getText("black"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
      if ( printerState.colorLevel[CYAN] )
      {
         strcat(message,getText("cyan"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
      if ( printerState.colorLevel[MAGENTA] )
      {
         strcat(message,getText("magenta"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
      if ( printerState.colorLevel[YELLOW] )
      {
         strcat(message,getText("yellow"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
      if ( printerState.colorLevel[LCYAN] )
      {
         strcat(message,getText("lcyan"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
      if ( printerState.colorLevel[LMAGENTA] )
      {
         strcat(message,getText("lmagenta"));
         strcat(message," ");
         strcat(message,getText("inkLow"));
         strcat(message,"\n");
      }
   }
}

static void waitLayout()
{
   int tw,th;
   int mw, mh;
   int y;
   int maxw= 0;
   
   getTextSize(waitTitle, &tw, &th);
   getTextSize(waitMessage, &mw, &mh);
   maxw = tw;
   if ( maxw < mw ) maxw = mw;
   y = 5;
   waitTitle->resize(5,y,maxw,th);
   y += 5 + th;
   waitMessage->resize(5,y,maxw,mh);
    y += 5 + mh;
  
   waitWin->size(maxw+ 10, y);
}

void make_waitWin()
{
   if ( device == NULL )
   {
      make_MsgWin(getText("noDevice"));
      return;
   }
   waitWin = new Fl_Window(10,10);
   waitWin->set_modal();
   waitWin->label(prgName);

   waitTitle = new Fl_Box(5, 5, 10, 10, getText(mainTitleLB));
   waitTitle->box(FL_FLAT_BOX);
   waitTitle->color(7);
   waitTitle->labelsize(18);
   waitTitle->labelcolor(1);
   waitTitle->align(FL_ALIGN_CLIP);
 
   waitMessage = new Fl_Box(5, 5, 10, 10, getText("WaitStatus"));
   if ( !useGL )
      waitMessage->box(FL_FLAT_BOX);
   waitMessage->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
   waitWin->end();

   waitLayout();
   setGlWin(waitWin);
   waitWin->show();
   getPrinterStatus(device, refreshFunction, &printerState);
   delete waitWin;
   
   /* look for the informations */
   if ( printerState.communicationError )
   {
      make_MsgWin(getText("msgCommError"));
      return;
   }
   buildMessage();
   make_MsgWin(getText(message));

}
