/* Open-Gl functions 
 */
 
int useGL        = 0;
int normalButton = 0;

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include "fltkGl.h"

#if HAVE_GL
#include <FL/gl.h>
#include <FL/fl_draw.H>

static void shiny_down_box(int x1, int y1, int w1, int h1, Fl_Color c);
static void shiny_flat_box(int x, int y1, int w, int h, Fl_Color c);
static void invisible_box(int x, int y, int w, int h, Fl_Color c);
static void shiny_up_box(int x1, int y1, int w1, int h1, Fl_Color c);

/* these function must be in fl_draw.h */
extern void fl_thin_up_box(int x1, int y1, int w1, int h1, Fl_Color c);
extern void fl_thin_down_box(int x1, int y1, int w1, int h1, Fl_Color c);
extern void fl_up_box(int x1, int y1, int w1, int h1, Fl_Color c);
extern void fl_down_box(int x1, int y1, int w1, int h1, Fl_Color c);

#endif

void setButton(Fl_Button *bt)
{
   Fl_Boxtype t = FL_PLASTIC_UP_BOX;
   if ( useGL ) return;
   if ( normalButton )
     t = FL_UP_BOX;
   bt->box(t);
}

void setGlWin(Fl_Window *win)
{
#if HAVE_GL
  if ( !useGL )
  {
     if ( normalButton )
     {
        Fl::set_boxtype(FL_UP_BOX, fl_thin_up_box,3,3,6,6);
        Fl::set_boxtype(FL_DOWN_BOX, fl_thin_down_box,3,3,6,6);
     }
     else
     {
        Fl::set_boxtype(FL_UP_BOX, fl_up_box,3,3,6,6);
        Fl::set_boxtype(FL_DOWN_BOX, fl_down_box,3,3,6,6);
     }  
     Fl::set_boxtype(FL_FLAT_BOX, fl_rectf, 0,0,0,0);
     win->box(FL_FLAT_BOX);
  }
  else
  {
     Fl::set_boxtype(FL_UP_BOX, shiny_up_box,3,3,6,6);
     Fl::set_boxtype(FL_DOWN_BOX, shiny_down_box,3,3,6,6);
     Fl::set_boxtype(SHINY_BOX, shiny_flat_box, 0,0,0,0);
     Fl::set_boxtype(INVISIBLE_BOX, invisible_box, 0,0,0,0);
     win->box(SHINY_BOX);
  }
#endif
}


#if HAVE_GL

static uchar color[8][3] = {
  {255,255,255},
  {0,0,0},
  {128,136,146},
  {145,145,145},
  {121,128,128},
  {255,255,255},
  {137,143,145},
  {115,115,115}
};

static int thickness = 2;
static Fl_Color pcolor;

static uchar C[8][3]; // actual colors for current button

static void calc_color(Fl_Color c) {
  uchar r[3];
  pcolor = c;
  Fl::get_color(c,r[0],r[1],r[2]);
  for (int x = 0; x<8; x++) for (int y=0; y<3; y++) {
    int i = r[y]-166+color[x][y];
    if (i<0) i = 0; else if (i>255) i = 255;
    C[x][y] = i;
  }
}

static void shiny_up_box(int x1, int y1, int w1, int h1, Fl_Color c) {
  if (c != pcolor) calc_color(c);
  int x = x1+1;
  int y = Fl_Window::current()->h()-(y1+h1-1);
  int w = w1-2;
  int h = h1-2;
  gl_start();

  // left edge:
  glBegin(GL_POLYGON);
  glColor3ub(C[0][0],C[0][1],C[0][2]);
  glVertex2i(x,y);
  glVertex2i(x+thickness,y+thickness);
  glColor3ub(C[3][0],C[3][1],C[3][2]);
  glVertex2i(x+thickness,y+h-thickness);
  glVertex2i(x,y+h);
  glEnd();

  // top edge:
  glBegin(GL_POLYGON);
  glVertex2i(x,y+h);
  glVertex2i(x+thickness,y+h-thickness);
  glColor3ub(C[2][0],C[2][1],C[2][2]);
  glVertex2i(x+w-thickness,y+h-thickness);
  glVertex2i(x+w,y+h);
  glEnd();

  // right edge:
  glColor3ub(C[1][0],C[1][1],C[1][2]);
  glBegin(GL_POLYGON);
  glVertex2i(x+w-thickness,y+thickness);
  glVertex2i(x+w,y+thickness);
  glVertex2i(x+w,y+h);
  glVertex2i(x+w-thickness,y+h-thickness);
  glEnd();

  // bottom edge:
  glBegin(GL_POLYGON);
  glVertex2i(x,y);
  glVertex2i(x+w,y);
  glVertex2i(x+w,y+thickness);
  glVertex2i(x+thickness,y+thickness);
  glEnd();

  glBegin(GL_POLYGON);
  glColor3ub(C[4][0],C[4][1],C[4][2]);
  glVertex2i(x+thickness,y+thickness);
  glColor3ub(C[5][0],C[5][1],C[5][2]);
  glVertex2i(x+w-thickness,y+thickness);
  glColor3ub(C[6][0],C[6][1],C[6][2]);
  glVertex2i(x+w-thickness,y+h-thickness);
  glColor3ub(C[7][0],C[7][1],C[7][2]);
  glVertex2i(x+thickness,y+h-thickness);
  glEnd();

  gl_finish();
  fl_color(FL_BLACK);
  fl_rect(x1,y1,w1,h1);
}

static void shiny_down_box(int x1, int y1, int w1, int h1, Fl_Color c) {
  if (c != pcolor) calc_color(c);
  int x = x1+1;
  int y = Fl_Window::current()->h()-(y1+h1-1);
  int w = w1-2;
  int h = h1-2;
  gl_start();

  // left edge:
  glColor3ub(C[1][0],C[1][1],C[1][2]);
  glBegin(GL_POLYGON);
  glVertex2i(x,y);
  glVertex2i(x+thickness,y+thickness);
  glVertex2i(x+thickness,y+h-thickness);
  glVertex2i(x,y+h);
  glEnd();

  // top edge:
  glBegin(GL_POLYGON);
  glVertex2i(x,y+h);
  glVertex2i(x+thickness,y+h-thickness);
  glVertex2i(x+w-thickness,y+h-thickness);
  glVertex2i(x+w,y+h);
  glEnd();

  // bottom edge:
  glBegin(GL_POLYGON);
  glColor3ub(C[0][0],C[0][1],C[0][2]);
  glVertex2i(x+thickness,y+thickness);
  glVertex2i(x,y);
  glColor3ub(C[1][0],C[1][1],C[1][2]);
  glVertex2i(x+w,y);
  glVertex2i(x+w-thickness,y+thickness);
  glEnd();

  // right edge:
  glBegin(GL_POLYGON);
  glVertex2i(x+w-thickness,y+thickness);
  glVertex2i(x+w,y);
  glColor3ub(C[2][0],C[2][1],C[2][2]);
  glVertex2i(x+w,y+h);
  glVertex2i(x+w-thickness,y+h-thickness);
  glEnd();

  // inside:
  glBegin(GL_POLYGON);
  glColor3ub(C[4][0],C[4][1],C[4][2]);
  glVertex2i(x+thickness,y+thickness);
  glColor3ub(C[5][0],C[5][1],C[5][2]);
  glVertex2i(x+w-thickness,y+thickness);
  glColor3ub(C[6][0],C[6][1],C[6][2]);
  glVertex2i(x+w-thickness,y+h-thickness);
  glColor3ub(C[7][0],C[7][1],C[7][2]);
  glVertex2i(x+thickness,y+h-thickness);
  glEnd();

  gl_finish();
  fl_color(FL_BLACK);
  fl_rect(x1,y1,w1,h1);
}

static void shiny_flat_box(int x, int y1, int w, int h, Fl_Color c) {
  if (c != pcolor) calc_color(c);
  int y = Fl_Window::current()->h()-(y1+h);
  gl_start();
  glBegin(GL_POLYGON);
  glColor3ub(C[4][0],C[4][1],C[4][2]);
  glVertex2i(x,y);
  glColor3ub(C[5][0],C[5][1],C[5][2]);
  glVertex2i(x+w,y);
  glColor3ub(C[6][0],C[6][1],C[6][2]);
  glVertex2i(x+w,y+h);
  glColor3ub(C[7][0],C[7][1],C[7][2]);
  glVertex2i(x,y+h);
  glEnd();
  gl_finish();
}

static void invisible_box(int x, int y, int w, int h, Fl_Color c) {
  fl_clip(x,y,w,h);
  Fl_Window *W = Fl_Window::current();
  fl_draw_box(W->box(),0,0,W->w(),W->h(),c);
  fl_pop_clip();
}
#endif
